// $RCSfile: CheckModule.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia;


import java.io.*;
import java.util.*;

import com.meyling.principia.module.*;
import com.meyling.principia.io.*;


/**
 * This class checks Principia Mathematica II qedeq modules for errors.
 *
 * @version $Revision: 1.4 $
 * @author  Michael Meyling
 */
public class CheckModule {


    /**
     * Writes calling convention to <code>System.err</code>.
     */
    private static void info() {
        System.err.println();
        System.err.println("Program description:");
        System.err.println();
        System.err.println("This program checks Principia Mathematica II modules for errors. Such a module");
        System.err.println("contains mathematical knowledge in a checkable form. This programm tries to");
        System.err.println("load the modules by the given addresses. All neccassary other modules");
        System.err.println("(\"IMPORTS\") are also loaded. If any loaded module doesn't pass the");
        System.err.println("verification check, the result is negative and the position and reason of the");
        System.err.println("problem is given (path to local copy \":\" line number \":\" column number).");
        System.err.println("Any new module is first copied into a local file buffer and then loaded");
        System.err.println("into memory. Further attempts to load the module lead to a search in the memory");
        System.err.println("and next in the local file buffer.");
        System.err.println("If you want to create a fresh copy, you have to delete the file buffer manually");
        System.err.println("(Normal location: \"com/meyling/principia/buffer\").");
        System.err.println();
        System.err.println("For more information on Principia Mathematica II look at:");
        System.err.println("http://www.meyling.com/principia/principia.html");
        System.err.println();
        System.err.println();
        System.err.println("Calling convention:");
        System.err.println();
        System.err.println("com.meyling.principia.CheckModule (-<option>)* (<module_address>)+");
        System.err.println();
        System.err.println("option:");
        System.err.println("\t-h\twrites this text");
        System.err.println("\t-l n\tsets the log level to n (2 is default, 0 means no logging)");
        System.err.println();
        System.err.println("module_address:");
        System.err.println("\taddress of a module to check, e.g.:");
        System.err.println("http://www.meyling.com/principia/0_00_51/propaxiom_1.00.00_1.00.00.qedeq");
    }


    /**
     * Check modules by trying to load them.
     *
     * @param   args addresses of modules and options
     */
    public static void main(final String[] args) {

        final List address = new ArrayList();

        if (args.length == 0) {
            info();
            return;
        }
        ModuleContext.setLogLevel(2);
        for (int i = 0; i < args.length; i++) {
            if (args[i].startsWith("-")) {  // option
                final String option = args[i].substring(1).toLowerCase();
                if (option.equals("help") || option.equals("h")
                        || option.equals("?")) {
                    info();
                } else if (option.equals("log") || option.equals("l")) {
                    i++;
                    if (i >= args.length) {
                        System.err.println("\nOption parameter missing: " + option);
                        info();
                        return;
                    }
                    final String parameter = args[i];
                    final int n;
                    try {
                        n = Integer.parseInt(parameter);
                    } catch (Exception e) {
                        System.err.println("\nOption parameter is no non negative integer: "
                            + parameter);
                        info();
                        return;
                    }
                    try {
                        ModuleContext.setLogLevel(n);
                    } catch (Exception e) {
                        System.err.println("\nCouldn't set log level: "
                            + e.getMessage());
                        info();
                        return;
                    }
                } else {                    // unknown option
                    System.err.println("\nUnknown option: " + option);
                    info();
                    return;
                }
            } else {                        // no option, must be file name
                address.add(args[i]);
            }
        }
        for (int i = 0; i < address.size(); i++) {
            checkModule((String) address.get(i));
        }
    }


    /**
     * Check a module.
     *
     * @param   address  address of module
     * @return  is module ok?
     */
     public final static boolean checkModule(final String address) {
        try {
            ModuleContext.loadModule(address);
            System.out.println("checked:");
            System.out.println("  " + address);
            return true;
        } catch (IOException e) {
            System.err.println(e.getMessage());
        } catch (IllegalArgumentException e) {
            e.printStackTrace();    // TODO
            System.err.println(e.getMessage());
        } catch (ParsingException e) {
            System.err.println(new File(e.getAddress())
                .getAbsolutePath() + ":" + e.getRow() + ":"
                + e.getColumn() + ":");
            System.err.println(e.getDescription());
            System.err.println(e.getLine());
            final StringBuffer pointer = new StringBuffer(e.getColumn());
            for (int i = 0; i < e.getColumn() - 1; i++) {
                pointer.append(' ');   // TODO Q & D
            }
            pointer.append("^");
            System.err.println(pointer);
        }
        return false;
    }


}
