// $RCSfile: MakeLatexModule.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia;


import java.io.*;
import java.util.*;

import com.meyling.principia.module.*;
import com.meyling.principia.latex.*;
import com.meyling.principia.io.*;



/**
 * This class creates LaTeX files from Principia Mathematica II qedeq modules.
 *
 * @version $Revision: 1.3 $
 * @author  Michael Meyling
 */
public class MakeLatexModule {


    /**
     * Writes calling convention to <code>System.err</code>.
     */
    private static void info() {
        System.err.println();
        System.err.println("Program description:");
        System.err.println();
        System.err.println("This program creates LaTeX files from Principia Mathematica II modules.");
        System.err.println("Such a module contains mathematical knowledge in a checkable form. This ");
        System.err.println("programm tries to load the modules by the given addresses. All neccassary");
        System.err.println("other modules (\"IMPORTS\") are also loaded.");
        System.err.println("Any new module is first copied into a local file buffer and then loaded");
        System.err.println("into memory. Further attempts to load the module lead to a search in the memory");
        System.err.println("and next in the local file buffer.");
        System.err.println("If you want to create a fresh copy, you have to delete the file buffer manually");
        System.err.println("(Normal location: \"com/meyling/principia/buffer\").");
        System.err.println();
        System.err.println("The resulting LaTeX file is at the same place as the buffered module.");
        System.err.println();
        System.err.println("For more information on Principia Mathematica II look at:");
        System.err.println("http://www.meyling.com/principia/principia.html");
        System.err.println();
        System.err.println();
        System.err.println("Calling convention:");
        System.err.println();
        System.err.println("com.meyling.principia.MakeLatexModule (-<option>)* (<module_address>)+");
        System.err.println();
        System.err.println("option:");
        System.err.println("\t-h\twrites this text");
        System.err.println("\t-l n\tsets the log level to n (2 is default, 0 means no logging)");
        System.err.println();
        System.err.println("module_address:");
        System.err.println("\taddress of a module to create a html file from, e.g.:");
        System.err.println("http://www.meyling.com/principia/0_00_51/propaxiom_1.00.00_1.00.00.qedeq");
    }


    /**
     * Create LaTeX modules.
     *
     * @param   args addresses of modules and options
     */
    public static void main(final String[] args) {

        final List address = new ArrayList();

        if (args.length == 0) {
            info();
            return;
        }
        ModuleContext.setLogLevel(2);
        for (int i = 0; i < args.length; i++) {
            if (args[i].startsWith("-")) {  // option
                final String option = args[i].substring(1).toLowerCase();
                if (option.equals("help") || option.equals("h")
                        || option.equals("?")) {
                    info();
                } else if (option.equals("log") || option.equals("l")) {
                    i++;
                    if (i >= args.length) {
                        System.err.println("\nOption parameter missing: " + option);
                        info();
                        return;
                    }
                    final String parameter = args[i];
                    final int n;
                    try {
                        n = Integer.parseInt(parameter);
                    } catch (Exception e) {
                        System.err.println("\nOption parameter is no non negative integer: "
                            + parameter);
                        info();
                        return;
                    }
                    try {
                        ModuleContext.setLogLevel(n);
                    } catch (Exception e) {
                        System.err.println("\nCouldn't set log level: "
                            + e.getMessage());
                        info();
                        return;
                    }
                } else {                    // unknown option
                    System.err.println("\nUnknown option: " + option);
                    info();
                    return;
                }
            } else {                        // no option, must be file name
                address.add(args[i]);
            }
        }
        for (int i = 0; i < address.size(); i++) {
            makeLatexModule((String) address.get(i));
        }
    }


    /**
     * Create a LaTeX module.
     *
     * @param   address  address of module
     */
     public final static void makeLatexModule(final String address) {
        try {
            (new Module2Latex(address)).writeModule();
            return;
        } catch (IllegalArgumentException e) {
            System.err.println(e.getMessage());
        } catch (IOException e) {
            System.err.println(e.getMessage());
        } catch (ParsingException e) {
            System.err.println(new File(e.getAddress())
                .getAbsolutePath() + ":" + e.getRow() + ":"
                + e.getColumn() + ":");
            System.err.println(e.getDescription());
            System.err.println(e.getLine());
            final StringBuffer pointer = Utility.getSpaces(e.getColumn());
            pointer.append("^");
            System.err.println(pointer);
        }
        System.err.println("creation of LaTeX file failed for:");
        System.err.println("  " + address);
    }

}
