// $RCSfile: Abbreviation.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.logic.paragraph;


import com.meyling.principia.argument.*;
import com.meyling.principia.module.*;
import com.meyling.principia.logic.basic.*;


/**
 * Abbreviation. For example OR (@0 @1) := NOT(AND(@0 @1)).
 * The left side must have only one operator and the right side
 * couldn't have other pattern variables than the left side.
 *
 * @version $Revision: 1.3 $
 * @author  Michael Meyling
 */
public class Abbreviation extends AbstractArgumentList implements Argument,
        ParagraphCheck {

    /** &quot;left&quot; side of abbreviation */
    final private Formula left;

    /** &quot;left&quot; side of abbreviation */
    final private Formula right;


    /**
     * Constructs a definition..
     *
     * @param   arguments     contains left and right side of definition
     * @throws  ArgumentException    if there are not exactly two
     *          arguments that are instances of {@link Formula} or
     *          if the first argument does not have only pattern variables
     *          as arguments or
     *          if the second argument has other pattern variables than the
     *          first one
     */
    public Abbreviation(Argument[] arguments)
            throws ArgumentException {
        super(arguments);
        if (arguments.length != 2) {
            throw new ArgumentException(
                ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_NUMBER,
                ModuleCreator.getName(this.getClass())
                + ParagraphConstants.OPERATOR_WITH_TWO_ARGUMENTS);
        }
        if (!(arguments[0] instanceof Formula)) {
            throw new ArgumentException(
                ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ModuleCreator.getName(this.getClass())
                + ParagraphConstants.FIRST_ARGUMENT_TYPE
                + ParagraphConstants.FORMULA,
                arguments[0]);
        }
        if (!(arguments[0] instanceof Formula)) {
            throw new ArgumentException(
                ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ModuleCreator.getName(this.getClass())
                + ParagraphConstants.SECOND_ARGUMENT_TYPE
                + ParagraphConstants.FORMULA,
                arguments[1]);
        }
        this.left = (Formula) arguments[0];
        this.right = (Formula) arguments[1];
        final PatternVariables already = new PatternVariables();

        for (int i = 0; i < left.getArgumentSize(); i++) {
            if (!(left.getArgument(i) instanceof PatternVariable)) {
                throw new ArgumentException(
                    ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                    ModuleCreator.getName(this.getClass())
                    + ParagraphConstants.FIRST_ARGUMENT
                    + ParagraphConstants.DIFFERENT_PATTERN_VARIABLES,
                    left.getArgument(i));
            } else {
                final PatternVariable variable
                    = (PatternVariable) left.getArgument(i);
                if (already.contains(variable)) {
                    throw new ArgumentException(
                        ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                        ModuleCreator.getName(this.getClass())
                        + ParagraphConstants.FIRST_ARGUMENT
                        + ParagraphConstants.DIFFERENT_PATTERN_VARIABLES,
                        left.getArgument(i));
                } else {
                    already.add(variable);
                }
            }
        }
        if (!left.getPatternVariables().equals(right.getPatternVariables())) {
            throw new ArgumentException(
                ParagraphConstants.CODE_CONSTRUCTOR_ARGUMENT_CHECK_FAILED,
                ModuleCreator.getName(this.getClass())
                + ParagraphConstants.SECOND_ARGUMENT
                + ParagraphConstants.SAME_PATTERN_VARIABLES);
        }

    }


    /**
     * Adds abbreviation to module.
     *
     * @param   module   module
     * @param   label     label of paragraph
     */
    public final void check(final Module module, final String label) {
        module.addAbbreviation(label, this, module);

    }


    /** @inherited */
    public final Argument create(final Argument[] arguments)
            throws ArgumentException {
        return new Abbreviation(arguments);
    }


    /** @inherited */
    public final String toString() {
        return "Abbreviation " + getArgument(0) + " := " + getArgument(1);
    }

}
