// $RCSfile: ParagraphCreator.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.logic.paragraph;


import com.meyling.principia.argument.*;
import com.meyling.principia.logic.rule.*;
import com.meyling.principia.io.*;

import java.io.Reader;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;


/**
 * This class provides static methods for reading and
 * writing an {@link Argument}.
 *
 * @version $Revision: 1.2 $
 * @author  Michael Meyling
 */
public final class ParagraphCreator {

    // now follow the strings that represent operators

    /** string for a {@link ProofLine} */
    public static final String PARAGRAPH_PROOF_LINE
        = "LINE";

    /** string for a {@link ProofLineList} */
    public static final String PARAGRAPH_PROOF_LINE_LIST
        = "PROOF";

    /** string for an {@link Abbreviation} */
    public static final String PARAGRAPH_ABBREVIATION
        = "ABBREVIATION";

    /** string for an {@link Axiom} */
    public static final String PARAGRAPH_AXIOM
        = "AXIOM";

    /** string for a {@link Sentence} */
    public static final String PARAGRAPH_SENTENCE
        = "SENTENCE";

    /** string for a {@link LinkLabel} */
    public static final String PARAGRAPH_LINK_LABEL
        = "LABEL";

    /** string for a {@link Proposition} */
    public static final String PARAGRAPH_PROPOSITION
        = "PROPOSITION";

    /** string for a {@link Paragraph} */
    public static final String PARAGRAPH_PARAGRAPH
        = "PARAGRAPH";

    public static final String PARAGRAPH_RULE_DECLARATION
        = "DECLARERULE";


    /**
     * for converting a <code>String</code> into an
     * (Argument) <code>class</code>
     */
    private final static Map string2class = new HashMap();

    /**
     * for converting a (Argument) <code>class</code> into an
     * <code>String</code>
     */
    private final static Map class2string = new HashMap();

    /**
     * for converting an created Argument into an input file position
     */
    private final ParsingTable parsing = new ParsingTable();


    /**
     * Make attribut initialization.
     */
    static {
        // fill hash map
        string2class.put(PARAGRAPH_PROOF_LINE,
                         ProofLine.class);
        string2class.put(PARAGRAPH_PROOF_LINE_LIST,
                         ProofLineList.class);
        string2class.put(PARAGRAPH_ABBREVIATION,
                         Abbreviation.class);
        string2class.put(PARAGRAPH_AXIOM,
                         Axiom.class);
        string2class.put(PARAGRAPH_SENTENCE,
                         Sentence.class);
        string2class.put(PARAGRAPH_LINK_LABEL,
                         LinkLabel.class);
        string2class.put(PARAGRAPH_PROPOSITION,
                         Proposition.class);
        string2class.put(PARAGRAPH_PARAGRAPH,
                         Paragraph.class);
        string2class.put(PARAGRAPH_RULE_DECLARATION,
                         RuleDeclaration.class);

        // add all the basic staff from package
        //  com.meyling.principia.logic.rule
        string2class.putAll(RuleCreator.getMap());

        // build inverse mapping
        for (Iterator it = string2class.entrySet().iterator(); it.hasNext(); ) {
            Map.Entry entry = (Map.Entry) it.next();
            class2string.put(entry.getValue(), entry.getKey());
        }

    }


    /**
     * Get the mapping of Strings to operator classes. The returned Map
     * should not be modified.
     *
     * @return  mapping of known Strings, do not change this Map
     */
    public final static Map getMap() {
        return string2class;
    }


    /**
     * Get name of known operator class.
     *
     * @param   clazz   look for name of this class
     * @return  name of operator class
     * @throws  IllegalArgumentException    if operator class <code>c</code>
     *          is unknown
     */
    public final static String getName(final Class clazz)
            throws IllegalArgumentException {
        final String result;
        try {
            result = (String) class2string.get(clazz);
        } catch (Exception e) {
            throw new IllegalArgumentException(e.toString());
        }
        if (result == null) {
            throw new IllegalArgumentException(clazz.getName()
                + ArgumentConstants.UNKNOWN_CLASS);
        }
        return result;
    }


    /**
     * Create an {@link Argument} out of an {@link TextInput} object.
     *
     * @param   input   source to read from
     * @param   parsing maps {@link Argument}s into {@link Position}s
     * @return  created argument
     * @throws  ParsingException    if creation failed
     */
    public final static Argument readArgument(final TextInput input,
            final ParsingTable parsing)
            throws ParsingException {
        return ArgumentCreator.readArgument(input, string2class, parsing);
    }


    /**
     * Write an {@link Argument} to an {@link Output} object.
     *
     * @param   output   destination to write to
     * @param   argument this object will be written
     * @throws  IllegalArgumentException    if writing failed
     */
    public final static void writeArgument(final Output output,
            final Argument argument) throws IllegalArgumentException {
        ArgumentCreator.writeArgument(output, argument, class2string);
    }


    /**
     * Write an {@link Argument} to an <code>String</code>.
     *
     * @param   argument this object will be written
     * @return  created string
     * @throws  IllegalArgumentException    if writing failed
     */
    public final static String writeArgument(
            final Argument argument) throws IllegalArgumentException {
        return ArgumentCreator.writeArgument(argument, class2string);
    }


}
