// $RCSfile: Particularization.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.logic.rule;


import com.meyling.principia.argument.*;
import com.meyling.principia.module.*;
import com.meyling.principia.logic.basic.*;
import com.meyling.principia.logic.paragraph.*;


/**
 * Implemention of making a conclusion special.
 * <p>
 * <code> A0 => A1       </code><p>
 * <code> x0 free in A0  </code><p>
 * <code> x0 not in A1   </code><p>
 * <code>----------------</code><p>
 * <code> E x0 (A0) => A1</code><p>
 *
 * @version $Revision: 1.5 $
 * @author  Michael Meyling
 */
public class Particularization extends AbstractArgumentList
        implements Argument, Rule {

    /** minimum version required for using this rule */
    private final static Version version;

    /** proof line of <code>A0 => A1</code>*/
    private final int n;

    /** quantify with this subject variable */
    private final SubjectVariable var;


    /**
     * Initialize version information.
     */
    static {
        try {
            version = new Version(
                    new Argument[] {new Text("1.00.00")});
        } catch (ArgumentException e) {
            throw new IllegalArgumentException(e.toString());
        }
    }


    /**
     * Constructs a particularization rule.
     *
     * @param   arguments
     *          <ul>
     *            <li>first argument is a proof line number
     *              to operate on, this proof line must be an implication
     *            </li>
     *            <li>second list entry must be a subject variable that
     *              dosn't occur in the first part of the implication,
     *              but is a free variable of
     *              the second part
     *            </li>
     *          </ul>
     * @throws  ArgumentException    if the first argument is not
     *  an instance of {@link Counter} or is not bigger than zero, or
     *  if the second argument is no instance of {@link SubjectVariable}
     */
    public Particularization(final Argument[] arguments)
            throws ArgumentException {
        super (arguments);
        if (arguments.length != 2) {
            throw new ArgumentException(
                RuleConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_NUMBER,
                RuleCreator.getName(this.getClass())
                + RuleConstants.OPERATOR_WITH_TWO_ARGUMENTS);
        }
        if (!(arguments[0] instanceof Counter)) {
            throw new ArgumentException(
                RuleConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                RuleCreator.getName(this.getClass())
                + RuleConstants.FIRST_ARGUMENT_TYPE
                + RuleConstants.NUMBER,
                arguments[0]);
        }
        this.n = ((Counter) arguments[0]).getNumber() - 1;
        if (n < 0) {
            throw new ArgumentException(
                RuleConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                RuleCreator.getName(this.getClass())
                + RuleConstants.FIRST_ARGUMENT_TYPE
                + RuleConstants.POSITIVE_INTEGER_EXPECTED,
                arguments[0]);
        }

        if (!(arguments[1] instanceof SubjectVariable)) {
            throw new ArgumentException(
                RuleConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                RuleCreator.getName(this.getClass())
                + RuleConstants.SECOND_ARGUMENT_TYPE
                + RuleCreator.getName(SubjectVariable.class),
                arguments[1]);
        }
        this.var = (SubjectVariable) arguments[1];
        // check if subject variable has a number
        if (!(var.getArgument(0) instanceof Counter)) {
            throw new ArgumentException(
                RuleConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                RuleCreator.getName(this.getClass())
                + RuleConstants.SECOND_ARGUMENT
                + RuleConstants.MUST_HAVE_COUNTER,
                var.getArgument(0));
        }
    }


    /**
     * Check if proof could be extended with formula because of
     * modus ponens.
     *
     * @param   module   module
     * @param   proofLines  proof lines
     * @param   position number of proof lines that could be used
     *                    in the rule
     * @param   formula  formula to add
     * @throws  ArgumentException    if addition is not possible
     *          because the proof line numbers are not allowed (e.g.
     *          between 1 and the minimum of <code>position</code>
     *          and current maximum proof line number)
     *          or the proof line number is no {@link Implication}
     *          or the first formula of the implication has already the
     *           new subject variable
     *          or the second formula of the implication dosn't have
     *           the new subject variable as a free one
     *          or the resulting formula is not equal to
     *          <code>formula</code>
     *          or a version conflict occurs
     */
    public final void check(final Module module,
            final ProofLineList proofLines,
            final int position,
            final Formula formula) throws ArgumentException {

        if (version.greaterThan(module.getRuleVersion())) {
            throw new ArgumentException(
                RuleConstants.CODE_METHOD_ARGUMENT_CHECK_FAILED,
                RuleCreator.getName(this.getClass())
                    + " " + RuleConstants.RULE_VERSION_CONFLICT
                    + version.getText(),
                this);
        }
        if (n < 0 || n >= proofLines.getArgumentSize() || n >= position) {
            throw new ArgumentException(
                RuleConstants.CODE_METHOD_ARGUMENT_CHECK_FAILED,
                RuleCreator.getName(this.getClass())
                    + RuleConstants.PROOF_LINES_OUT_OF_RANGE,
                this.getArgument(0));
        }
        if (!(proofLines.getArgument(n).getArgument(0) instanceof Implication)) {
            throw new ArgumentException(
                RuleConstants.CODE_METHOD_ARGUMENT_CHECK_FAILED,
                RuleCreator.getName(this.getClass())
                    + RuleConstants.PROOF_LINE_NO_IMPLICATION,
                this,
                proofLines.getArgument(n).getArgument(0));
        }
        final Implication impl = (Implication) proofLines
            .getArgument(n).getArgument(0);
        final Formula first = (Formula) impl.getArgument(0);
        final Formula second = (Formula) impl.getArgument(1);
        if (!first.getFreeSubjectVariables().contains(var)
                || (second.getSubjectVariables().contains(var))) {
            throw new ArgumentException(
                RuleConstants.CODE_METHOD_ARGUMENT_CHECK_FAILED,
                RuleCreator.getName(this.getClass())
                    + RuleConstants
                    .IMPLICATON_FIRST_NOT_FREE_OR_SECOND_ALREADY,
                this,
                proofLines.getArgument(n).getArgument(0));

        }
        final Implication result = new Implication(
            new Argument[] {
                new ExistentialQuantifier(
                    new Argument[] {
                        var,
                        first
                    }
                ),
                second
            }
        );
        if (!result.equals(formula)) {
            throw new ArgumentException(
                RuleConstants.CODE_METHOD_ARGUMENT_CHECK_FAILED,
                RuleCreator.getName(this.getClass())
                    + RuleConstants.RESULT_NOT_EQUAL_WITH_EXPECTED,
                this,
                proofLines.getArgument(n).getArgument(0));
        }
    }


    /** @inherited */
    public ProofLineList extendWithout(final Module module,
            final ProofLineList proofLines, final int position)
            throws UnsupportedOperationException {
        throw new UnsupportedOperationException (
            "this rule is neccessary: "
            + RuleCreator.getName(this.getClass()));
    }


    /** @inherited */
    public int[] getProofLines() {
        return new int[] {this.n};
    }


    /** @inherited */
    public Rule changeProofLines(final int[] mapping) {
        try {
            return (Rule) create(new Argument[] {
                new Counter(mapping[n] + 1), var});
        } catch (ArgumentException e) {
            throw new IllegalArgumentException(e.toString());
        } catch (IndexOutOfBoundsException e) {
            throw new IllegalArgumentException(e.toString());
        }
    }


    /**
     * Check this rule could be declared.
     *
     * @param   module      the module context
     * @param   declaration the declaration that shall declare this rule
     * @throws  IllegalArgumentException    if an programming error occured
     * @throws  ArgumentException    if declaration failed
     */
    public final static void checkDeclaration(final Module module,
        final RuleDeclaration declaration) throws ArgumentException {
        if (declaration.getArgumentSize() > 2) {
            throw new ArgumentException(
                RuleConstants.CODE_METHOD_ARGUMENT_CHECK_FAILED,
                ParagraphCreator.getName(RuleDeclaration.class)
                    + RuleConstants.OPERATOR_WITH_TWO_ARGUMENTS,
                declaration);
        }
    }


    /** @inherited */
    public final Version getVersion() {
        return this.version;
    }


    /** @inherited */
    public final Argument create(final Argument[] arguments)
            throws ArgumentException {
        return new Particularization(arguments);
    }


    /** @inherited */
    public final String toString() {
        return "Particularization " + getArgument(0) + getArgument(0);
    }


}
