// $RCSfile: Rule.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.logic.rule;


import com.meyling.principia.argument.*;
import com.meyling.principia.module.*;
import com.meyling.principia.logic.basic.*;
import com.meyling.principia.logic.paragraph.*;


/**
 * Every logical rule must implement this interface. Each rule makes it possible
 * to extend a proof.
 *
 * @version $Revision: 1.4 $
 * @author  Michael Meyling
 */
public interface Rule extends Argument {


    /**
     * Check if proof could be extended with formula (using the arguments of rule).
     *
     * @param   module    the module context
     * @param   proofLines the proof lines
     * @param   position number of proof lines that could be used
     *                    in the rule
     * @param   formula  formula to add
     * @throws  ArgumentException    if addition is not possible
     */
    public void check(Module module, ProofLineList proofLines,
            int position, Formula formula)
        throws ArgumentException;


    /**
     * Return proof lines that could replace the proof line
     * <code>position</code>.
     *
     * @param   module      the belonging module
     * @param   proofLines  proof lines
     * @param   position    referenced proof line (starting with 0)
     *                       in <code>proofLines</code>
     * @return  proof lines that could substitute the single
     *           proof line <code>position</code>
     * @throws  IllegalArgumentException  if extension is impossible
     *           or other problems occured
     *           if the module was checked before, this exception
     *           could only occur if there is an programming error
     * @throws  UnsupportedOperationException if this rule couldn't
     *           be reduced at all because it is a basic rule
     */
    public ProofLineList extendWithout(final Module module,
            final ProofLineList proofLines, final int position)
            throws IllegalArgumentException;


    /**
     * Get proof line numbers that are used to derive the new formula.
     *
     * @return  numbers of involved proof lines
     */
    public int[] getProofLines();


    /**
     * Get proof line numbers that are used to derive the new formula.
     *
     * @param   mapping     array that maps old proof line numbers (index) to
     *                      new ones (value)
     * @return  same rule but with changed numbers of involved proof lines
     * @throws  IllegalArgumentException    if a needed value is -1
     */
    public Rule changeProofLines(int[] mapping);


    /**
     * Get version of this rule.
     *
     * @return  rule version
     */
    public Version getVersion();


}
