// $RCSfile: ModuleContext.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.module;


import com.meyling.principia.argument.*;
import com.meyling.principia.logic.basic.*;
import com.meyling.principia.logic.paragraph.*;
import com.meyling.principia.io.*;

import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;
import java.net.URL;
import java.net.URLConnection;
import java.net.MalformedURLException;


/**
 * This class provides static access methods for loading modules.
 *
 * @version $Revision: 1.13 $
 * @author  Michael Meyling
 */
public final class ModuleContext {

    /** version of this kernel */
    private final static Version kernelVersion;

    /** maximum version of supported rules */
    private final static Version ruleVersion;

    /**
     * already loaded modules
     */
    private final static Map modules = new HashMap();

    /**
     * current logging level:
     *   0 = no logging,
     *   1 = elementary events,
     *   2 = more events
     */
    private static int logLevel = 1;

    /** thread for watching loading process */
    private final static PeekTextInput peek = new PeekTextInput();

    /**
     * Make attribut initialization.
     */
    static {
        // Initialize version information.
        try {
            kernelVersion = new Version("0.00.51");
            ruleVersion = new Version("1.02.00");
        } catch (ArgumentException e) {
            throw new IllegalArgumentException(e.toString());
        }
        // TODO fill hash map with initial values
        peek.setDaemon(true);
        peek.start();
        log(1, "this is pmii, version " + getKernelVersion());
        log(1, "  see \"http://www.meyling.com/principia/principia.html\" for more information");
        log(1, "  supports rules till version " + getRuleVersion());
    }


    /**
     * Get maximum version of supported rules.
     *
     * @return  rule version
     */
    public final static String getRuleVersion() {
        return ModuleContext.ruleVersion.getText();
    }


    /**
     * Get version of this kernel.
     *
     * @return  kernel version
     */
    public final static String getKernelVersion() {
        return ModuleContext.kernelVersion.getText();
    }


// TODO not neccessary any longer because peek is now a daemon thread
/*
    public final static void exit() {
        peek.stopPeek();
    }
*/

// TODO not neccessary any longer
    /**
     * Check a module.
     *
     * @param   address  address of module
     * @return  is module ok?
     */
/*
     public final static boolean checkModule(final String address) {
        try {
            loadModule(address);
            return true;
        } catch (IllegalArgumentException e) {
            log(2, e);
            throw e;
        } catch (ParsingException e) {
            log(2, e);
        }
        return false;
     }

*/

    /**
     * Remove double and unused lines of all propositions of a module and
     * save modified local buffer back. The module in module cache is not
     * modified.
     *
     * @param   address  address of module
     * @throws  IOException if module file couldn't be read or written
     * @throws  ParsingException if
     */
     public final static boolean compress(final String address)
            throws IOException, ParsingException {
        final ModuleAddress moduleAddress = new ModuleAddress(address);
        final Module module;
        try {
            module = (Module) loadModule(moduleAddress).copy();
        } catch (IllegalArgumentException e) {
            log(1, e);
            throw new IOException(e.getMessage());
        }
        try {
            module.setModuleAddress(moduleAddress);
            final Proposition[] propositions = module.getPropositions();
            for (int i = 0; i < propositions.length; i++) {
                propositions[i].compress();
            }
            module.check();
            final StringBuffer buffer = new StringBuffer();
            final Output output = new Output(buffer);
            ModuleWriter.writeModule(module, output);
            Utility.saveFile(moduleAddress.getLocalName(), buffer);
            /* replace old module in memory buffer */
            modules.remove(moduleAddress);
            modules.put(moduleAddress, module);
        } catch (IOException e) {
            log(3, e);
            throw new IllegalArgumentException(e.getMessage());
        } catch (IllegalArgumentException e) {
            log(1, e);                  // programming error
            throw e;
        } catch (ArgumentException e) { // could only occur if
                                        // module wasn't checked
            log(3, e);
            throw new IllegalArgumentException(e.toString());
        } catch (ParsingException e) {
            log(3, e);
            throw e;
        }
        return false;
     }


    /**
     * Reduce module file to module file that uses only basic rules.
     * (Basic rules have the rule version &quot;1.00.00&quot;.)
     * The resulting file will be saved at the same directory
     * in the <strong>file buffer</code> where the original resides.
     *
     * @param   address  address of module
     * @throws  IOException if writing the resulting file is not possible
     *                  (e.g. because there is already a module file of
     *                  that name)
     * @throws  ParsingException    if there were a parsing problem for
     *          the referenced (or one indirectly connected) module
     * @throws  IllegalArgumentException if something unexpected happens
     *                  (e.g. a programming error occurs)
     */
     public final static boolean reduceToBasicRules(final String address)
            throws IOException, ParsingException {
        final ModuleAddress moduleAddress = new ModuleAddress(address);
        final Version version;
        final ModuleAddress newAddress;
        try {
            version =  new Version("1.00.00");
            newAddress = new ModuleAddress(moduleAddress,
                moduleAddress.getName(),
                moduleAddress.getModuleVersion(), version);
            final File file = new File(newAddress.getLocalName());
            if (file.exists()) {
                throw new IOException(
                    ModuleConstants.FILE_EXISTS_ALREADY + "\n  "
                    + newAddress.getLocalName());
            }
        } catch (ArgumentException e) {
            log(3, e);
            throw new IllegalArgumentException(e.getMessage());
        } catch (IOException e) {
            log(3, e);
            throw e;
        }
        try {
            final Module module = (Module) loadModule(moduleAddress).copy();
            module.setModuleAddress(moduleAddress);
            module.check();
            final Proposition[] propositions = module.getPropositions();
            for (int i = 0; i < propositions.length; i++) {
                propositions[i].reduceRuleVersion(module, version);
            }
            module.setRuleVersion(version);
            module.setModuleAddress(newAddress);
            if (!newAddress.equals(module.getModuleAddress())) {
                throw new IllegalArgumentException("addresses not equal: "
                    + newAddress + " != " + module.getModuleAddress());
            }
            module.check();
            final StringBuffer buffer = new StringBuffer();
            final Output output = new Output(buffer);
            ModuleWriter.writeModule(module, output);
            Utility.saveFile(module.getModuleAddress().getLocalName(),
                buffer);
        } catch (IOException e) {
            log(3, e);
            throw e;
        } catch (IllegalArgumentException e) {
            log(1, e);
            throw e;
        } catch (ArgumentException e) {
            log(1, e);
            throw new IllegalArgumentException(e.getMessage());
        } catch (ParsingException e) {
            log(3, e);
            throw e;
        }
        return false;
     }


    /**
     * Get a certain module.
     *
     * @param  address  address of module
     * @return  wanted module
     * @throws  IOException    if loading was not successfull
     * @throws  ParsingException if loading the module lead to
     *           an parsing error
     */
    public final synchronized static Module loadModule(
            final String address)
            throws ParsingException, IOException {
            return loadModule(new ModuleAddress(address));
    }


    /**
     * Get a certain module.
     *
     * @param  address  address of module
     * @return  wanted module
     * @throws  IOException    if loading was not successfull
     * @throws  ParsingException if loading the module lead to
     *           an parsing error
     */
    public final synchronized static Module loadModule(
            final ModuleAddress moduleAddress)
            throws ParsingException, IOException {
        // search in already loaded modules
        ModuleContext.peek.setInput(null);      // stop peeking
        if (moduleAddress.getRuleVersion().greaterThan(
                ModuleContext.ruleVersion)) {
            log(2, "loading:");
            log(2, "  " + moduleAddress);
            throw new IOException(
                "loading failed because rule version is not supported by this kernel: "
                + moduleAddress.getRuleVersion().getText());
        }
        if (modules.containsKey(moduleAddress)) {
            log(2, "loading from memory:");
            log(2, "  " + moduleAddress);
            return (Module) modules.get(moduleAddress);
        }

        // search in local file buffer
        try {
            final Module mod = loadLocalModule(moduleAddress);
            modules.put(moduleAddress, mod);
            return mod;
        } catch (ModuleFileNotFoundException e) {     // file not found
        } catch (IOException e) {
            log(3, e);
            throw e;
        }

        // try loading url directly
        if (makeLocalCopy(moduleAddress)) {
            try {
                final Module mod = loadLocalModule(moduleAddress);
                modules.put(moduleAddress, mod);
                return mod;
            } catch (IOException e) {
                log(3, e);
                throw e;
            } catch (ModuleFileNotFoundException e) {
                log(1, e);
                throw new IOException(e.getMessage());
            }
        }
        log(3, ModuleConstants.MODULE_NOT_FOUND + moduleAddress);
        throw new IOException(
            ModuleConstants.MODULE_NOT_FOUND + moduleAddress);
     }



    /**
     * Load a certain module.
     *
     * @param  module   this is the current module
     * @param  spec     specification of wanted module
     * @return wanted module
     * @throws ArgumentException if loading was not successfull
     * @throws ParsingException if loading another module lead to
     *          an parsing error
     */
    public final synchronized static Module loadModule(final Module module,
            final Specification spec)
            throws ArgumentException, ParsingException {

        ModuleContext.peek.setInput(null);  // stop peeking
        final ModuleAddress[] modulePaths
            = ModuleAddress.getModulePaths(module, spec);
        // search in already loaded modules
        for (int i = 0; i < modulePaths.length; i++) {
            if (modules.containsKey(modulePaths[i])) {
                log(2, "loading from memory:");
                log(2, "  " + modulePaths[i]);
                return (Module) modules.get(modulePaths[i]);
            }
        }

        // search in local file buffer
        for (int i = 0; i < modulePaths.length; i++) {
            try {
                final Module mod = loadLocalModule(modulePaths[i]);
                modules.put(modulePaths[i], mod);
                return mod;
            } catch (ModuleFileNotFoundException e) {  // file not found
            } catch (IOException e) {
                throw new ArgumentException(
                    ModuleConstants.CODE_METHOD_ARGUMENT_CHECK_FAILED,
                    e.toString(),
                    spec.getArgument(3).getArgument(i).getArgument(0));
            } catch (IllegalArgumentException e) {
                throw new ArgumentException(
                    ModuleConstants.CODE_METHOD_ARGUMENT_CHECK_FAILED,
                    e.toString(),
                    spec.getArgument(3).getArgument(i).getArgument(0));
            }
        }

        // try loading url directly
        for (int i = 0; i < modulePaths.length; i++) {
            try {
                if (makeLocalCopy(modulePaths[i])) {
                    final Module mod = loadLocalModule(modulePaths[i]);
                    modules.put(modulePaths[i], mod);
                    return mod;
                }
             } catch (IOException e) {
                throw new ArgumentException(
                    ModuleConstants.CODE_METHOD_ARGUMENT_CHECK_FAILED,
                    e.toString(),
                    spec.getArgument(3).getArgument(i).getArgument(0));
             } catch (ModuleFileNotFoundException e) {
                throw new ArgumentException(
                    ModuleConstants.CODE_METHOD_ARGUMENT_CHECK_FAILED,
                    e.toString(),
                    spec.getArgument(3).getArgument(i).getArgument(0));
             } catch (IllegalArgumentException e) {
                throw new ArgumentException(
                    ModuleConstants.CODE_METHOD_ARGUMENT_CHECK_FAILED,
                    e.toString(),
                    spec.getArgument(3).getArgument(i).getArgument(0));
             }
        }
        throw new ArgumentException(
            ModuleConstants.CODE_METHOD_ARGUMENT_CHECK_FAILED,
            ModuleConstants.MODULE_NOT_FOUND,
            spec);
    }


    /**
     * Load a module.
     *
     * @param   address   module address
     * @return  loaded module
     * @throws  ModuleFileNotFoundException    if local file was not found
     * @throws  IOException if loading failed
     * @throws  ParsingException if loading the module lead to
     *           an parsing error
     */
    public final static Module loadLocalModule(
            final ModuleAddress moduleAddress)
            throws ModuleFileNotFoundException, IOException, ParsingException  {
        final StringBuffer buffer = new StringBuffer();
        final File file = new File(moduleAddress.getLocalName());
        if (file.canRead()) {
            Utility.loadFile(file, buffer);
            final TextInput in = new TextInput(buffer,
                moduleAddress.getAddress(),
                moduleAddress.getLocalName());
            log(2, "loading from local file buffer:");
            log(2, "  " + moduleAddress);
            final Argument argument;
            final TextInput oldIn = ModuleContext.peek.getInput();
            ModuleContext.peek.setInput(in);
            try {
                argument = (new ModuleCreator(in)).readArgument();
                if (!(argument instanceof Module)) {
                    throw new ParsingException(
                        ModuleConstants.CODE_PARSER_UNEXPECTED_ARGUMENT,
                        ModuleConstants.NO_FOLLOWING_TEXT_ALLOWED,
                        in.getRow(),
                        in.getColumn(),
                        in.getLine(),
                        in.getLocalAddress());
                }
                in.skipWhiteSpace();
                if (in.getChar() != Character.MAX_VALUE) {
                    throw new ParsingException(
                        ModuleConstants
                            .CODE_PARSER_NO_FURTHER_ARGUMENTS_EXPECTED,
                        ModuleConstants.NO_FOLLOWING_TEXT_ALLOWED,
                        in.getRow(),
                        in.getColumn(),
                        in.getLine(),
                        in.getLocalAddress());
                }
                try {
                    ((Module) argument).setModuleAddress(moduleAddress);
                } catch (ArgumentException e) {
                    throw new ParsingException(
                        ModuleConstants
                            .CODE_METHOD_ARGUMENT_CHECK_FAILED,
                        e.getDescription(),
                        0,
                        0,
                        e.getArgument().toString(),
                        moduleAddress.getLocalName());
                }
            } finally {
                ModuleContext.peek.setInput(oldIn);
            }
            return (Module) argument;
        } else {
            throw new ModuleFileNotFoundException("file not found");
        }
    }


    /**
     * Make local copy of a module.
     *
     * @param   address   module address
     * @return  success?
     * @throws  IOException    if address was malformed
     */
    private final static boolean makeLocalCopy(
            final ModuleAddress moduleAddress)
            throws IOException {
        log(2, "creating local copy of:");
        log(2, "  " + moduleAddress);
        FileOutputStream out = null;
        InputStream in = null;
        try {
            final URLConnection connection = moduleAddress.getURL().openConnection();
            in = connection.getInputStream();
            if (!moduleAddress.getURL().equals(connection.getURL())) {
                throw new FileNotFoundException("\"" + moduleAddress.getURL()
                    + "\" was substituted by " + "\"" + connection.getURL()
                    + "\" from server");
            }
            {   // assure existence of directories
                String dir = moduleAddress.getLocalName();
                final File f = new File(dir).getParentFile();
                f.mkdirs();
            }
            out = new FileOutputStream(moduleAddress.getLocalName());
            final byte[] buffer = new byte[4096];
            int bytesRead;
            if ((bytesRead = in.read(buffer)) != -1) {
                int i = 0;
                final String first = new String(buffer);
                while (i < first.length() && Character.isWhitespace(first.charAt(i))) {
                    i++;
                }
                if (i + 6 >= first.length()
                        || !first.substring(i, i + 6).equals("MODULE")) {
                    throw new FileNotFoundException(
                        "this is no valid \".qedeq\" module file");
                }
                out.write(buffer, 0, bytesRead);
            } else {
                throw new IOException("module file has 0 bytes length");
            }
            // continue writing
            while ((bytesRead = in.read(buffer)) != -1) {
                out.write(buffer, 0, bytesRead);
            }
        } catch (IOException e) {
            log(2,  ".. failed because: " + e.toString());
            try {
                out.close();
            } catch (Exception ex) {}
            try {
                new File(moduleAddress.getLocalName()).delete();
            } catch (Exception ex) {}
            return false;
        } finally {
            try {
                out.close();
            } catch (Exception e) {};
            try {
                in.close();
            } catch (Exception e) {};
        }
//        check for module ###########
        return true;
    }


    /**
     * Set logging level:
     *   0 = no logging,
     *   1 = elementary events,
     *   2 = more events
     *
     * @param   level   logging level
     */
    public final static void setLogLevel(final int level) {
        if (level < 0) {
            throw new IllegalArgumentException(
                "only non negative log levels allowed");
        }
        ModuleContext.logLevel = level;
    }


    /**
     * Log message object with certain level to <code>System.out</code>:
     *   1 = elementary events,
     *   2 = more events.
     * Logging only occurs if current logging level is set
     * to value equal or below <code>level</code>.
     * (See {@link #setLogLevel}).
     *
     * @param   level   logging level
     * @param   object  this object will be logged
     */
    public final static void log(final int level, final Object object) {
        if (level <= 0) {
            throw new IllegalArgumentException(
                "only positive (non zero) log levels allowed");
        }
        if (level <= ModuleContext.logLevel) {
            if (object != null) {
                System.out.println(object.toString());
            } else {
                System.out.println("null");
            }
        }
    }




    private static class ModuleFileNotFoundException extends Exception {


        ModuleFileNotFoundException(final String message) {
            super(message);
        }

    }



    /**
     * This thread watches the reading process of a {@link TextInput} object.
     * After creating a <code>PeekTextInput</code> instance and starting
     * the thread by calling the <code>start</code>-Method, an object
     * to watch could be set by calling <code>setInput</code>.
     * It logs the reading position after 10 seconds on log level 1 (if
     * there is a {@link TextInput} object.
     */
    private static class PeekTextInput extends Thread {

        private TextInput reader = null;

        private long maximum;

        private boolean stopped = false;

        private Thread myThread = null;

        /**
         * Set object to watch (could be <code>null</code>.
         *
         * @param   reader  look at this object
         */
        private final void setInput(final TextInput reader) {
            synchronized (this) {   // inform waiting threads
                this.reader = reader;
                if (reader != null) {
                    this.maximum = reader.getMaximumPosition();
                }
                if (myThread != null) {
                    myThread.interrupt();
                }
                else {
                    this.notifyAll();
                }
            }
        }

        /**
         * Get the current {@link TextInput} object.
         *
         * @return  watched object (could be <code>null</code>).
         */
        private final TextInput getInput() {
            return reader;
        }


       /**
        * Main loop.
        */
        public void run() {
            myThread = Thread.currentThread();
            while (!stopped) {
                synchronized (this) {
                    try {
                        this.wait(10000);
                    } catch (InterruptedException e) {
                        continue;
                    }
                    if (reader == null) {
                        continue;
                    }
                    final long position = reader.getPosition();
                    String percent = "" + (position * 100 / maximum);
                    switch (percent.length()) {
                    case 1: percent = "  " + percent;
                        break;
                    case 2: percent = " " + percent;
                        break;
                    }
                    String row = "" + reader.getRow();
                    switch (row.length()) {
                    case 1: row = "        " + row;
                        break;
                    case 2: row = "       " + row;
                        break;
                    case 3: row = "      " + row;
                        break;
                    case 4: row = "     " + row;
                        break;
                    case 5: row = "    " + row;
                        break;
                    case 6: row = "   " + row;
                        break;
                    case 7: row = "  " + row;
                        break;
                    case 8: row = " " + row;
                        break;
                    }
                    log(2, "  " + percent + "% read, line number "
                        + row + " of:");
                    log(2, "  " + reader.getAddress());
                    if (position == maximum) {
                        reader = null;
                    }
                }
            }
        }


        /**
         * Call this method to stop this thread.
         */
        public void stopPeek() {
            synchronized (this) {   // inform waiting threads
                stopped = true;
                if (myThread != null) {
                    myThread.interrupt();
                }
                else {
                    this.notifyAll();
                }
            }
        }

    }


}
